package org.rollproject.rvrp.model.problem;

import java.io.Serializable;
import java.text.ParseException;
import java.util.ArrayList;

import javax.xml.bind.annotation.XmlType;

import org.rollproject.rvrp.model.problem.*;



@XmlType(namespace="org.rollproject.rvrp.model.problem", name = "delivery", propOrder = {"ID", "routeId", "pallets", "breaks", "drivingTime", "unloadingTime", "customerId", "jobId"})
public class Delivery implements Serializable{
	
	private static final long serialVersionUID = 1L;	
	private int ID = -1;
	private static int id = 0;	
	private ArrayList<Pallet> pallets;
	/**
	 * generator only adds breaks after driving so if drive home (last delivery ) 
	 * takes us over there will be a break before unloading even though there is no unloading
	 *  
	 */
	private ArrayList<TimeWindow> breaks;
	private TimeWindow drivingTime;
	private TimeWindow unloadingTime;
	private int routeId = -1;
	private int customerId = -1;
	private int jobId = -1;
	
	public Delivery() {
		id++;
		ID = id;
	}	
	public int getID() {
		return ID;
	}
	public void setID(int iD) {
		ID = iD;
	}
	public ArrayList<Pallet> getPallets() {
		if(pallets == null){
			pallets = new ArrayList<Pallet>();
		}
		return pallets;
	}
	public void setPallets(ArrayList<Pallet> pallets) {
		this.pallets = pallets;
	}	
	public ArrayList<TimeWindow> getBreaks() {
		if(breaks == null){
			breaks = new ArrayList<TimeWindow>();
		}
		return breaks;
	}	
	public void setBreaks(ArrayList<TimeWindow> breaks) {
		this.breaks = breaks;
	}
	public TimeWindow getDrivingTime() {
		if(drivingTime == null){
			drivingTime = new TimeWindow();
		}
		return drivingTime;
	}
	public void setDrivingTime(TimeWindow drivingTime) {
		this.drivingTime = drivingTime;
	}
	public TimeWindow getUnloadingTime() {
		if(unloadingTime == null){
			unloadingTime = new TimeWindow();

			//if no unloading ie last "delivery" to depot
			unloadingTime.setStartTime(getDrivingTime().getEndTime());
			unloadingTime.setEndTime(getDrivingTime().getEndTime());
		}
		return unloadingTime;
	}
	public void setUnloadingTime(TimeWindow unloadingTime) {
		this.unloadingTime = unloadingTime;
	}		
	public int getCustomerId() {
		return customerId;
	}
	public void setCustomerId(int customerId) {
		this.customerId = customerId;
	}
	public int getJobId() {
		return jobId;
	}
	public void setJobId(int jobId) {
		this.jobId = jobId;
	}	
	public int getRouteId() {
		return routeId;
	}
	public void setRouteId(int routeId) {
		this.routeId = routeId;
	}
	
//	
	/**
	 * the end time of the delivery 
	 * always the end of unloading
	 * if no unloading set ie last delivery to depot then getunloading returns the end driving time  
	 * breaks are only added prior to inloading 
	 * 
	 * @return
	 */
	public String getEndTime(){
		if(getDrivingTime().getEndTime().equals("")){
			System.err.println("No Driving Time Set for Delivery");
			System.exit(-1);
		}
		return getUnloadingTime().getEndTime();			
	}
	
	@Override
	public String toString() {
		
		String str = "";
		if(getPallets() == null || getPallets().size() == 0){
			str = "Empty Delivery Driving from " + getDrivingTime().toString();
			return str;
		}		
		str += "Delivery of " + getPallets().size() + " pallets : Driving : " + getDrivingTime().toString() + " Unloading " + getUnloadingTime().toString() + " Breaks - ";
		for(TimeWindow tw : getBreaks()){
			str += tw.toString() + "\t";
		}
		return str;
	}
}
