package org.rollproject.rvrp.model.problem;

import java.io.Serializable;
import java.util.ArrayList;

import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;


@XmlRootElement(namespace="org.rollproject.rvrp.model.problem", name = "solution")
@XmlType(namespace="org.rollproject.rvrp.model.problem", name = "solution", propOrder = { "problemID", "routes", "fitness", "problem"})
public class Solution implements Serializable{	 

	private static final long serialVersionUID = 1L;
	
	private int problemID;	
	private ArrayList<Route> routes = null;
	private double fitness = -1;
	
	
	private Problem problem = null;
	private transient ArrayList<Job> jobsRemaining = null;
	
		
	
	public Solution() {
		
	}
	
	public Solution(Problem problem) {
		this.problem = problem;
		setRoutes(new ArrayList<Route>());		
		setProblemID(problem.getID());
		jobsRemaining = new ArrayList<Job>();
		for(Job job : problem.getJobs()){
			jobsRemaining.add(job);
		}
	}
	
	public boolean hasJobsRemaining(){
		if(jobsRemaining != null && jobsRemaining.size() > 0){
			return true;
		}
		return false;
	}

	
	public Problem getProblem() {
		return problem;
	}

	public double getFitness() {		
		return fitness;
	}

	public ArrayList<Route> getOpenRoutes(){
		ArrayList<Route> openRoutes = new ArrayList<Route>();
		for(Route route : getRoutes()){
			if(route.isOpen()){
				openRoutes.add(route);
			}
			
		}			
		return openRoutes;
	}
	
	@XmlTransient
	public ArrayList<Job> getJobsRemaining() {
		return jobsRemaining;
	}

	
	public int getProblemID() {
		return problemID;
	}

	public void setProblemID(int problemID) {
		this.problemID = problemID;
	}

	public ArrayList<Route> getRoutes() {
		if(routes == null){
			routes = new ArrayList<Route>();
		}
		return routes;
	}

	public void setRoutes(ArrayList<Route> routes) {
		this.routes = routes;
	}

	public void setProblem(Problem problem) {
		this.problem = problem;
	}

	public void setJobsRemaining(ArrayList<Job> jobsRemaining) {
		this.jobsRemaining = jobsRemaining;
	}

	public void setFitness(double fitness) {
		this.fitness = fitness;
	}

	public Solution copy() {
		System.out.println("TODO solution copy");
		return null;
	}
	@Override
	public String toString() {
		String str = "";
		str += "Solution to problem " + problemID + " has fitness " + fitness;
		if(problem != null){
			str += "\r\n" + problem.toString();
		}
		for(Route route : routes){
			str += route.toString() + "\r\n";
		}
		return str;
	}
}
