package org.rollproject.rvrp.model.problem;

import java.io.Serializable;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;



@XmlType(namespace="org.rollproject.rvrp.model.problem", name="timewindow",propOrder={"startTime", "endTime"})
public class TimeWindow implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String startTime;
	private String endTime;
	public TimeWindow() {
		
	}
	
	public String getStartTime() {		
		return startTime;
	}
	public void setStartTime(String startTime) {
		this.startTime = startTime;
	}
	public String getEndTime() {		
		return endTime;
	}
	public void setEndTime(String endTime) {
		this.endTime = endTime;
	}	
	
	/**
	 * ===============================
	 *         Helper methods
	 * ===============================
	 */

	
	/**
	 * for a String representing time in "yyyyMMdd'T'HHmmss" return an integer representing the time in minutes since 1970 
	 * 
	 * @param timeString
	 * @return
	 * @throws ParseException
	 */
	public static int getDateInMinutes(String timeString) throws ParseException{
		DateFormat dateFormat = new SimpleDateFormat("yyyyMMdd'T'HHmmss");
		Date date = dateFormat.parse(timeString);
		long time = date.getTime();
		return (int)(time / 60000);
	}
		
	
	/**
	 * Returns String representing time in "yyyyMMdd'T'HHmmss" from time in minutes since 1970 
	 * @param mins
	 * @return
	 */
	public static String getDateString(int mins){
		DateFormat dateFormat = new SimpleDateFormat("yyyyMMdd'T'HHmmss");
		long milli = mins * 60000L;
		return dateFormat.format(new Date(milli));
	}


	public static int getLengthOfTimeInMinutes(String startTime, String endTime) throws Exception  {
		int start;
		int end;		
		try {
			start = getDateInMinutes(startTime);
			end = getDateInMinutes(endTime);
			if(end < start){
				throw new Exception("Invalid Time Window : End before Start");		
			}
		} catch (ParseException e) {
			throw new Exception("Invalid Time Window");			
		}		
		return end - start;		
	}
	
	public static int getLengthOfTimeWindowInMinutes(TimeWindow tw) throws Exception  {
		int start;
		int end;		
		try {
			start = getDateInMinutes(tw.getStartTime());
			end = getDateInMinutes(tw.getEndTime());
			if(end < start){
				throw new Exception("Invalid Time Window : End before Start");		
			}
		} catch (ParseException e) {
			throw new Exception("Invalid Time Window");			
		}		
		return end - start;		
	}
	
	@Override
	public String toString() {		
		return getStartTime() + " - " + getEndTime();
	}

	public boolean contains(String dt) {		
		try {
			int start = TimeWindow.getDateInMinutes(getStartTime());
			int end = TimeWindow.getDateInMinutes(getEndTime());
			int time = TimeWindow.getDateInMinutes(dt);
			if(start <= time && time <= end){
				return true;
			}
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	

	@XmlTransient
	public int getStartMinutes() throws ParseException{
		return getDateInMinutes(getStartTime());
	}
	@XmlTransient
	public int getEndMinutes() throws ParseException{
		return getDateInMinutes(getEndTime());
	}
	public void setStartMinutes(int minutes) {
		setStartTime(getDateString(minutes));
	}
	public void setEndMinutes(int minutes) {
		setEndTime(getDateString(minutes));		
	}
	@XmlTransient
	public int getDurationMinutes() throws ParseException {
		return getEndMinutes() - getStartMinutes();		
	}
}
