package org.rollproject.rvrp.model.problem;


import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.HashSet;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;



public class XmlParser{	
	
	public static void main(String[] args) throws Exception {
		String problemFilename = "p1003640.xml";		
		String companyDataFilename = "cd4.xml";
		Problem p = XmlParser.readProblem(problemFilename);
//		Solution s = readSolution(solutionFilename);
		CompanyData cd = readCompanyData(companyDataFilename);
		
		p = addLists(p, cd);
		
		System.out.println(p);
//		System.out.println(s);
	}
		
	private static Problem addLists(Problem problem, CompanyData companyData) throws Exception{
		HashSet<Company> allowedCompanies = new HashSet<Company>();		
		allowedCompanies.add(problem.getDepot());
		for(Job j : problem.getJobs()){
			allowedCompanies.add(j.getCompany());			
		}		
		ArrayList<ListElement> smallList = getReducedList(allowedCompanies, companyData.getDistanceList());		
		problem.setDistanceList(smallList);
		smallList = getReducedList(allowedCompanies, companyData.getTimeList());
		problem.setTimeList(smallList);
		return problem;
	}
	/**
	 * For a given list of ListElements reduces it to only those in the supplied company list
	 * 
	 * @param allowedCompanies
	 * @param elementList
	 * @return
	 */
	private static ArrayList<ListElement> getReducedList(
			HashSet<Company> allowedCompanies, ArrayList<ListElement> elementList) {
		ArrayList<ListElement> reducedList = new ArrayList<ListElement>();
		ArrayList<Integer> allowedIds = new ArrayList<Integer>();
		for(Company c : allowedCompanies){
			allowedIds.add(c.getID());
		}		
		for(ListElement le : elementList){						
			if(allowedIds.contains(le.getFromCompanyIndex()) && allowedIds.contains(le.getToCompanyIndex())){
				reducedList.add(le);
			}
		}
		return reducedList;
	}
	
	/**
	 * This Method reads the XML to the Solution.class  
	 * @param filename
	 * @return
	 * @throws Exception
	 */
	private static Solution readSolution(String filename) throws Exception{
		Solution solution = null;
		JAXBContext context = JAXBContext.newInstance(Solution.class);
	    Unmarshaller m = context.createUnmarshaller();	    
	    solution  = (Solution) m.unmarshal(new FileInputStream(filename));	    
		return solution;
	}		
	
	/**
	 * This Method reads the XML to the Problem.class that is annotated with @XmlRootElement(name="problem") 
	 * @param filename
	 * @param addLists 
	 * @return
	 * @throws Exception
	 */
	private static Problem readProblem(String problemFilename) throws Exception{
		Problem problem = null;
		JAXBContext context = JAXBContext.newInstance(Problem.class);
	    Unmarshaller m = context.createUnmarshaller();	    
	    problem = (Problem) m.unmarshal(new FileInputStream(problemFilename));		   
		return problem;
	}			
	
	private static CompanyData readCompanyData(String filename) throws Exception{
		CompanyData companyData = null;
		JAXBContext context = JAXBContext.newInstance(CompanyData.class);
	    Unmarshaller m = context.createUnmarshaller();	    
	    companyData = (CompanyData) m.unmarshal(new FileInputStream(filename));	
	    	    
		return companyData;
	}	

	public static Problem getProblem(String problemFilename, String companyDataSetFilename) throws Exception{
		Problem p = readProblem(problemFilename);
		CompanyData cd = readCompanyData(companyDataSetFilename);
		p.setDistanceList(cd.getDistanceList());
		p.setTimeList(cd.getTimeList());
		return p;
	}
}
